@@ -23,10 +23,10 @@ SOFTWARE.
 """
 from copy import copy
 from ctypes import get_errno
-from datetime import timedelta, datetime
+from datetime import timedelta
 from errno import ENOENT
 from os import strerror
-from typing import List
+from typing import Iterator, List, Optional, Union
 
 from .. import libgpiod
 
@@ -77,29 +77,29 @@ def chip_deleter(chip_struct: libgpiod.gpiod_chip):
 
 
 class shared_chip:
-    # pylint: disable=missing-function-docstring, bad-whitespace
-    def __init__(self, chip_struct: libgpiod.gpiod_chip = None):
+    # pylint: disable=missing-function-docstring
+    def __init__(self, chip_struct: Optional[libgpiod.gpiod_chip] = None):
         self._chip_struct = chip_struct
 
-    def get(self):
+    def get(self) -> Optional[libgpiod.gpiod_chip]:
         return self._chip_struct
 
-    def __del__(self):
+    def __del__(self) -> None:
         if self._chip_struct is not None:
             chip_deleter(self._chip_struct)
 
-    def __bool__(self):
+    def __bool__(self) -> bool:
         return self._chip_struct is not None
 
 
 class chip:
-    # pylint: disable=function-redefined, bad-whitespace
+    # pylint: disable=function-redefined
     def __init__(
         self,
-        device=None,
+        device: Optional[Union[int, str]] = None,
         how: int = chip.OPEN_LOOKUP,
-        chip_shared: shared_chip = None,
-    ):
+        chip_shared: Optional[shared_chip] = None,
+    ) -> None:
         """
         @brief Constructor. Creates an empty GPIO chip object or opens the chip
                 using chip.open.
@@ -112,7 +112,7 @@ class chip:
             c = chip("gpiochip0")
             c = chip("/dev/gpiochip0", chip.OPEN_BY_PATH)
         """
-        if bool(chip_shared):
+        if chip_shared is not None and bool(chip_shared):
             self._m_chip = chip_shared
             return
 
@@ -128,7 +128,9 @@ class chip:
             del chip
         """
 
-    def open(self, device, how: int = chip.OPEN_LOOKUP):
+    def open(
+        self, device: Union[int, str], how: int = chip.OPEN_LOOKUP
+    ) -> None:
         """
         @brief Open a GPIO chip.
 
@@ -142,11 +144,7 @@ class chip:
             chip.open("/dev/gpiochip0")
             chip.open(0, chip.OPEN_BY_NUMBER)
         """
-        if how == chip.OPEN_BY_NUMBER:
-            device = int(device)
-        else:
-            device = str(device)
-
+        device = str(device)
         func = open_funcs[how]
 
         chip_struct = func(device)
@@ -180,9 +178,7 @@ class chip:
         Usage:
             print(chip.name)
         """
-        self._throw_if_noref()
-
-        return self._m_chip.get().name
+        return self._throw_if_noref_and_get_m_chip().name
 
     @property
     def label(self) -> str:
@@ -194,9 +190,7 @@ class chip:
         Usage:
             print(chip.label)
         """
-        self._throw_if_noref()
-
-        return self._m_chip.get().label
+        return self._throw_if_noref_and_get_m_chip().label
 
     @property
     def num_lines(self) -> int:
@@ -208,9 +202,7 @@ class chip:
         Usage:
             print(chip.num_lines)
         """
-        self._throw_if_noref()
-
-        return self._m_chip.get().num_lines
+        return self._throw_if_noref_and_get_m_chip().num_lines
 
     def get_line(self, offset: int) -> line:
         """
@@ -223,12 +215,12 @@ class chip:
         Usage:
             l = chip.get_line(0)
         """
-        self._throw_if_noref()
-
         if offset >= self.num_lines or offset < 0:
             raise IndexError("line offset out of range")
 
-        line_struct = libgpiod.gpiod_chip_get_line(self._m_chip.get(), offset)
+        line_struct = libgpiod.gpiod_chip_get_line(
+            self._throw_if_noref_and_get_m_chip(), offset
+        )
         if line_struct is None:
             errno = get_errno()
             raise OSError(
@@ -248,9 +240,9 @@ class chip:
         Usage:
             l = chip.find_line("PIN_0")
         """
-        self._throw_if_noref()
-
-        line_struct = libgpiod.gpiod_chip_find_line(self._m_chip.get(), name)
+        line_struct = libgpiod.gpiod_chip_find_line(
+            self._throw_if_noref_and_get_m_chip(), name
+        )
         errno = get_errno()
         if line_struct is None and errno != ENOENT:
             raise OSError(
@@ -360,9 +352,11 @@ class chip:
     OPEN_BY_LABEL = 4
     OPEN_BY_NUMBER = 5
 
-    def _throw_if_noref(self):
-        if not bool(self._m_chip.get()):
+    def _throw_if_noref_and_get_m_chip(self) -> libgpiod.gpiod_chip:
+        _m_chip_get = self._m_chip.get()
+        if _m_chip_get is None or not bool(_m_chip_get):
             raise RuntimeError("object not associated with an open GPIO chip")
+        return _m_chip_get
 
 
 class line_request:
@@ -403,10 +397,12 @@ reqflag_mapping = {
 
 
 class line:
-    # pylint: disable=function-redefined, bad-whitespace
+    # pylint: disable=function-redefined
     def __init__(
-        self, line_struct: libgpiod.gpiod_line = None, owner: chip = chip()
-    ):
+        self,
+        line_struct: Optional[libgpiod.gpiod_line] = None,
+        owner: chip = chip(),
+    ) -> None:
         """
         @brief Constructor. Creates an empty line object.
 
@@ -434,9 +430,7 @@ class line:
         Usage:
             print(line.offset)
         """
-        self._throw_if_null()
-
-        return self._m_line.offset
+        return self._throw_if_null_and_get_m_line().offset
 
     @property
     def name(self) -> str:
@@ -448,9 +442,7 @@ class line:
         Usage:
             print(line.name)
         """
-        self._throw_if_null()
-
-        return self._m_line.name
+        return self._throw_if_null_and_get_m_line().name
 
     @property
     def consumer(self) -> str:
@@ -463,9 +455,7 @@ class line:
         Usage:
             print(line.consumer)
         """
-        self._throw_if_null()
-
-        return self._m_line.consumer
+        return self._throw_if_null_and_get_m_line().consumer
 
     @property
     def direction(self) -> int:
@@ -477,11 +467,10 @@ class line:
         Usage:
             print(line.direction == line.DIRECTION_INPUT)
         """
-        self._throw_if_null()
-
         return (
             self.DIRECTION_INPUT
-            if self._m_line.direction == libgpiod.GPIOD_LINE_DIRECTION_INPUT
+            if self._throw_if_null_and_get_m_line().direction
+            == libgpiod.GPIOD_LINE_DIRECTION_INPUT
             else self.DIRECTION_OUTPUT
         )
 
@@ -495,11 +484,9 @@ class line:
         Usage:
             print(line.active_state == line.ACTIVE_HIGH)
         """
-        self._throw_if_null()
-
         return (
             self.ACTIVE_HIGH
-            if self._m_line.active_state
+            if self._throw_if_null_and_get_m_line().active_state
             == libgpiod.GPIOD_LINE_ACTIVE_STATE_HIGH
             else self.ACTIVE_LOW
         )
@@ -515,9 +502,7 @@ class line:
         Usage:
             print(line.is_used)
         """
-        self._throw_if_null()
-
-        return self._m_line.used
+        return self._throw_if_null_and_get_m_line().used
 
     @property
     def is_open_drain(self) -> bool:
@@ -529,9 +514,7 @@ class line:
         Usage:
             print(line.is_open_drain)
         """
-        self._throw_if_null()
-
-        return self._m_line.open_drain
+        return self._throw_if_null_and_get_m_line().open_drain
 
     @property
     def is_open_source(self) -> bool:
@@ -543,9 +526,7 @@ class line:
         Usage:
             print(line.is_open_source)
         """
-        self._throw_if_null()
-
-        return self._m_line.open_source
+        return self._throw_if_null_and_get_m_line().open_source
 
     def request(self, config: line_request, default_val: int = 0):
         """
@@ -562,14 +543,14 @@ class line:
             # line.request(config)
             line.request(config, 1)
         """
-        self._throw_if_null()
+        _m_line = self._throw_if_null_and_get_m_line()
 
         conf = libgpiod.gpiod_line_request_config()
         conf.consumer = config.consumer
         conf.request_type = reqtype_mapping[config.request_type]
         conf.flags = 0
 
-        rv = libgpiod.gpiod_line_request(self._m_line, conf, default_val)
+        rv = libgpiod.gpiod_line_request(_m_line, conf, default_val)
         if rv:
             errno = get_errno()
             raise OSError(errno, strerror(errno), "error requesting GPIO line")
@@ -581,9 +562,7 @@ class line:
         Usage:
             line.release()
         """
-        self._throw_if_null()
-
-        libgpiod.gpiod_line_release(self._m_line)
+        libgpiod.gpiod_line_release(self._throw_if_null_and_get_m_line())
 
     @property
     def is_requested(self) -> bool:
@@ -595,9 +574,9 @@ class line:
         Usage:
             print(line.is_requested)
         """
-        self._throw_if_null()
-
-        return libgpiod.gpiod_line_is_requested(self._m_line)
+        return libgpiod.gpiod_line_is_requested(
+            self._throw_if_null_and_get_m_line()
+        )
 
     def get_value(self) -> int:
         """
@@ -608,9 +587,7 @@ class line:
         Usage:
             val = line.get_value()
         """
-        self._throw_if_null()
-
-        rv = libgpiod.gpiod_line_get_value(self._m_line)
+        rv = libgpiod.gpiod_line_get_value(self._throw_if_null_and_get_m_line())
         if rv == -1:
             errno = get_errno()
             raise OSError(
@@ -628,9 +605,9 @@ class line:
         Usage:
             line.set_value(1)
         """
-        self._throw_if_null()
-
-        rv = libgpiod.gpiod_line_set_value(self._m_line, val)
+        rv = libgpiod.gpiod_line_set_value(
+            self._throw_if_null_and_get_m_line(), val
+        )
         if rv:
             errno = get_errno()
             raise OSError(
@@ -652,9 +629,9 @@ class line:
             else:
                 print("Timeout")
         """
-        self._throw_if_null()
-
-        rv = libgpiod.gpiod_line_event_wait(self._m_line, timeout)
+        rv = libgpiod.gpiod_line_event_wait(
+            self._throw_if_null_and_get_m_line(), timeout
+        )
         if rv < 0:
             errno = get_errno()
             raise OSError(errno, strerror(errno), "error polling for events")
@@ -675,12 +652,12 @@ class line:
             else:
                 print("Timeout")
         """
-        self._throw_if_null()
+        _m_line = self._throw_if_null_and_get_m_line()
 
         event_buf = libgpiod.gpiod_line_event()
         event = line_event()
 
-        rv = libgpiod.gpiod_line_event_read(self._m_line, event_buf)
+        rv = libgpiod.gpiod_line_event_read(_m_line, event_buf)
         if rv < 0:
             errno = get_errno()
             raise OSError(errno, strerror(errno), "error reading line event")
@@ -705,9 +682,9 @@ class line:
         Usage:
             fd = line.event_get_fd()
         """
-        self._throw_if_null()
-
-        ret = libgpiod.gpiod_line_event_get_fd(self._m_line)
+        ret = libgpiod.gpiod_line_event_get_fd(
+            self._throw_if_null_and_get_m_line()
+        )
 
         if ret < 0:
             errno = get_errno()
@@ -788,9 +765,10 @@ class line:
     ACTIVE_LOW = 1
     ACTIVE_HIGH = 2
 
-    def _throw_if_null(self):
+    def _throw_if_null_and_get_m_line(self) -> libgpiod.gpiod_line:
         if self._m_line is None:
             raise RuntimeError("object not holding a GPIO line handle")
+        return self._m_line
 
 
 class line_event:
@@ -808,7 +786,7 @@ class line_event:
 class line_bulk:
     # pylint: disable=function-redefined
     # pylint: disable=missing-function-docstring
-    def __init__(self, lines: List[line] = None):
+    def __init__(self, lines: Optional[List[line]] = None) -> None:
         """
         @brief Constructor. Creates a empty line_bulk or from a list of lines.
 
@@ -822,7 +800,7 @@ class line_bulk:
         """
         self._m_bulk = lines if lines is not None else []
 
-    def __del__(self):
+    def __del__(self) -> None:
         """
         @brief Destructor
 
@@ -830,7 +808,7 @@ class line_bulk:
             del bulk
         """
 
-    def append(self, new_line: line):
+    def append(self, new_line: line) -> None:
         """
         @brief Add a line to this line_bulk object.
 
@@ -928,7 +906,9 @@ class line_bulk:
         """
         self._m_bulk.clear()
 
-    def request(self, config: line_request, default_vals: List[int] = None):
+    def request(
+        self, config: line_request, default_vals: Optional[List[int]] = None
+    ) -> None:
         """
         @brief Request all lines held by this object.
 
@@ -962,7 +942,7 @@ class line_bulk:
             self.release()
             raise error
 
-    def release(self):
+    def release(self) -> None:
         """
         @brief Release all lines held by this object.
 
@@ -992,7 +972,7 @@ class line_bulk:
 
         return values
 
-    def set_values(self, values: List[int]):
+    def set_values(self, values: List[int]) -> None:
         """
         @brief Set values of all lines held by this object.
 
@@ -1063,7 +1043,7 @@ class line_bulk:
         """
         return libgpiod.GPIOD_LINE_BULK_MAX_LINES
 
-    def __iter__(self) -> [].__iter__():
+    def __iter__(self) -> Iterator[line]:
         """
         @brief Iterator for iterating over lines held by line_bulk.
 
@@ -1077,7 +1057,7 @@ class line_bulk:
         if self.empty:
             raise RuntimeError("line_bulk not holding any GPIO lines")
 
-    def _to_line_bulk(self, bulk: libgpiod.gpiod_line_bulk):
+    def _to_line_bulk(self, bulk: libgpiod.gpiod_line_bulk) -> None:
         for it in self._m_bulk:
             # pylint: disable=protected-access
             bulk.add(it._m_line)
@@ -1121,15 +1101,13 @@ class line_iter:
             print("{}: {}".format(l.offset, l.name))
     """
 
-    def __init__(self, owner: chip):
+    def __init__(self, owner: chip) -> None:
         self._chip = owner
 
         self._iter = None
 
-    def __iter__(self):
-        self._iter = libgpiod.gpiod_line_iter(
-            self._chip._m_chip.get()
-        ).__iter__()
+    def __iter__(self) -> Iterator[libgpiod.gpiod_line]:
+        self._iter = iter(libgpiod.gpiod_line_iter(self._chip._m_chip.get()))
         if self._iter is None:
             errno = get_errno()
             raise OSError(
@@ -1139,4 +1117,7 @@ class line_iter:
         return self
 
     def __next__(self) -> line:
-        return line(self._iter.__next__(), self._chip)
+        if self._iter is not None:
+            return line(next(self._iter), self._chip)
+
+        raise StopIteration
